import time
import logging
import consts


logger = logging.getLogger(consts.LOGGER_NAME)


def clear_none_in_dict(d, recursive=True):
    clear_keys = []
    for k, v in d.items():
        if v is None:
            clear_keys.append(k)
        if isinstance(v, dict) and recursive:
            clear_none_in_dict(v, True)
    for k in clear_keys:
        del d[k]
    return d


def bound(val, min_val, max_val):
    return min(max(val, min_val), max_val)


def log_run_time(action):
    def decorator(func):
        def inner_func(*args, **kwargs):
            start = time.time()
            result = func(*args, **kwargs)
            logger.info("action [%s] cost [%f]", action, time.time() - start)
            return result
        return inner_func
    return decorator



class SingletonMeta(type):

    def __init__(cls, name, bases, attrs):
        cls.__instance = None
        super().__init__(name, bases, attrs)

    def __call__(cls, *args, **kwargs):
        if cls.__instance is None:
            cls.__instance = super().__call__(*args, **kwargs)
        return cls.__instance




class EnumMeta(type):

    def __init__(cls, name, bases, attrs):
        super().__init__(name, bases, attrs)
        vals = set()
        for k, v in attrs.items():
            if k.startswith("__"):
                continue
            if v in vals:
                raise Exception("value [%s] duplicated" % v)
            vals.add(v)
        cls.__vals = vals

    def __call__(cls, *args, **kwargs):
        raise Exception("can not construct Enum instance")

    def __contains__(cls, x):
        return x in cls.__vals



import setuptools
import etools as package


setuptools.setup(
    name="etools",
    version=package.__version__,
    author="sheng_xc",
    author_email="sheng_xc@126.com",
    description="some useful tools",
    long_description="some useful tools",
    long_description_content_type="text/markdown",
    url="",
    packages=setuptools.find_packages(),
    install_requires=[
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.5'
)
